/*
 *  ircd-ratbox: A slightly useful ircd.
 *  m_quit.c: Makes a user quit from IRC.
 *
 *  Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 *  Copyright (C) 1996-2002 Hybrid Development Team
 *  Copyright (C) 2002-2005 ircd-ratbox development team
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
 *  USA
 *
 *  $Id: m_quit.c 26094 2008-09-19 15:33:46Z androsyn $
 */

#include "stdinc.h"
#include "struct.h"
#include "client.h"
#include "ircd.h"
#include "send.h"
#include "parse.h"
#include "modules.h"
#include "s_conf.h"
#include "match.h"

static int m_quit(struct Client *, struct Client *, int, const char **);
static int mr_quit(struct Client *, struct Client *, int, const char **);
static int ms_quit(struct Client *, struct Client *, int, const char **);

struct Message quit_msgtab = {
	"QUIT", 0, 0, 0, MFLG_SLOW | MFLG_UNREG,
	{{mr_quit, 0}, {m_quit, 0}, {ms_quit, 0}, mg_ignore, mg_ignore, {m_quit, 0}}
};

mapi_clist_av1 quit_clist[] = { &quit_msgtab, NULL };

DECLARE_MODULE_AV1(quit, NULL, NULL, quit_clist, NULL, NULL, "$Revision: 26094 $");


static int
mr_quit(struct Client *client_p, struct Client *source_p, int parc, const char *parv[])
{
	exit_client(client_p, source_p, source_p, "Client Quit");
	return 0;
}

/* Function based upon the strip color functions found in
 * charybdis (http://svn.atheme.org/charybdis/trunk/src/irc_string.c)
 * ircd-hybrid7-bg (http://sourceforge.net/projects/hybrid7-bg)
 * and xchat (http://xchat.org/)
 */
static int
has_url(const char *p)
{
	static char buf[REASONLEN + 1];
	char *r, *q = buf;
	int w, h;
	w = h = 0;
	do switch (*p)
	{
		case '\003': /* khaled's shit */
			p++;
			if (isdigit(p[0]))
			{
				p++;
				if (isdigit(p[0]))
					p++;
				if (p[0] == ',' && isdigit(p[1]))
				{
					p += 2;
					if(isdigit(p[0]))
						p++;
				}
			}
			break;
		case '\033': /* ansi shits */
			p++;
			if (*p == '[' && p++)
			{
				r = strchr(p, 'm');
				if (r) p = r + 1;
			}
			break;
		case '\002': /* bold */
		case '\037': /* underline */
		case '\007': /* beep */
		case '\017': /* reset */
		case '\026': /* reverse */
		case '\035': /* italics */
			p++;
			break;
		default:
			*q++ = ToLower(*p++); break;
	} while (*p);

	if (strstr(buf, "www")) return 1;
	if (strstr(buf, "http")) return 1;
	return 0;
}

/*
** m_quit
**      parv[0] = sender prefix
**      parv[1] = comment
*/
static int
m_quit(struct Client *client_p, struct Client *source_p, int parc, const char *parv[])
{
	char *comment = LOCAL_COPY_N((parc > 1 && parv[1]) ? parv[1] : client_p->name, REASONLEN);
	char reason[REASONLEN + 1];

	source_p->flags |= FLAGS_NORMALEX;

	if(ConfigFileEntry.client_exit && comment[0])
	{
		rb_snprintf(reason, sizeof(reason), "Quit: %s", comment);
		comment = reason;
	}

	if(!IsOper(source_p) &&
		((source_p->localClient->firsttime + ConfigFileEntry.anti_spam_exit_message_time) >
		rb_current_time() || has_url(comment)))
	{
		exit_client(client_p, source_p, source_p, "Client Quit");
		return 0;
	}

	exit_client(client_p, source_p, source_p, comment);

	return 0;
}

/*
** ms_quit
**      parv[0] = sender prefix
**      parv[1] = comment
*/
static int
ms_quit(struct Client *client_p, struct Client *source_p, int parc, const char *parv[])
{
	char *comment = LOCAL_COPY_N((parc > 1 && parv[1]) ? parv[1] : client_p->name, REASONLEN);

	source_p->flags |= FLAGS_NORMALEX;
	exit_client(client_p, source_p, source_p, comment);

	return 0;
}
