/*
 *  m_operlist.c: List all (global) opers on the network or on a servermask
 *                provided.
 *
 *  Copyright 2003 by W. Campbell
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met:
 *
 *  1.Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  2.Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *  3.The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 *  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 *  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 *  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 *  $Id$
 *
 * Revision 1:  OPERLIST, list all operators on all servers or on servers
 *              on the given mask
 * Revision 2:  ratbox 1.5.x support by Alex
 * Revision 3:  ratbox 2.0.x support by Jeremy
 * Revision 4:  Ported for ratbox 3.0.x by Narf
 */

#include "stdinc.h"
#include "ratbox_lib.h"
#include "struct.h"
#include "client.h"
#include "ircd.h"
#include "numeric.h"
#include "s_log.h"
#include "s_serv.h"
#include "send.h"
#include "match.h"
#include "hash.h"
#include "parse.h"
#include "modules.h"
#include "s_newconf.h"

static int mo_operlist(struct Client *, struct Client *, int parc, const char *parv[]);

#ifndef STATIC_MODULES

struct Message operlist_msgtab = {
	"OPERLIST", 0, 0, 0, MFLG_SLOW,
	{mg_unreg, mg_not_oper, mg_ignore, mg_ignore, mg_ignore, {mo_operlist, 1}}
};

mapi_clist_av1 operlist_clist[] = { &operlist_msgtab, NULL };

DECLARE_MODULE_AV1(operlist, NULL, NULL, operlist_clist, NULL, NULL, "$Revision: 4 $");

#endif

/*
** mo_operlist
**      parv[0] = sender prefix
**      parv[1] = server mask
*/
static int
mo_operlist(struct Client *client_p, struct Client *source_p, int parc, const char *parv[])
{

	int count = 0;
	struct Client *target_p;
	rb_dlink_node *ptr;
	const char *smatch = NULL;

	if(IsOperSpy(source_p))
	{
		if(parc < 2 || EmptyString(parv[1]))
		{
			report_operspy(source_p, "OPERLIST", NULL);
		}
		else
		{
			report_operspy(source_p, "OPERLIST", parv[1]);
			smatch = parv[1];
		}

		RB_DLINK_FOREACH(ptr, global_client_list.head)
		{
			target_p = ptr->data;

			if(!IsClient(target_p) || !IsOper(target_p) || IsService(target_p))
				continue;

			if (IsOper(target_p))
			{

				if (smatch != NULL && match(smatch, target_p->servptr->name) == 0)
					continue;

				sendto_one(source_p, ":%s %d %s p :[%c] %s (%s@%s) on %s",
					   me.name, RPL_STATSDEBUG, source_p->name,
					   IsAdmin(target_p) ? 'A' : 'O',
					   target_p->name, target_p->username, target_p->host,
					   target_p->servptr->name);
				count++;

			}
		}
	}

	sendto_one(source_p, ":%s %d %s p :%d OPER(s)", me.name, RPL_STATSDEBUG, source_p->name, count);
	return 0;
}
